<?php

namespace App\Http\Controllers;

use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\Rule;

class UserController extends Controller
{
    // Muestra el listado de todos los usuarios
    public function index()
    {
        // Con listado dinámino, tomando datos desde la DB
        // Con el constructor de consultas de laravel
        // $users = DB::table('users')->get();

        // dd($users);  // Sirve para parar el proceso y mostar el resultado por pantalla

        // Con eloquent
        $users = User::all();

        // Con listado estático
        /*
        if (request()->has('empty')) {
            $users = [];
        } else {
            $users = [
                'Joel', 'Ellie', 'Tess','Tommy', 'Bill'
            ];
        }*/

        $title = "Listado de usuarios";

        // Forma alternativa para llamar a la vista
        /*
        return view('users.index')
            ->with('users', User::all())
            ->with('title', 'Listado de Usuarios');
        */

        return view('users.index', compact('title', 'users'));
    }
/*
    //  Otra forma de para la función show
    public function show($id)
    {
        $user = User::find($id);

         Condicional para evitar que el usuario busque un registro inexistente
        if ($user == null){
            return view('errors.404');
        }
        return view('users.show', compact('user'));
    }
*/

    // Muestra el detalle del usuario seleccionado
    public function show(User $user){
        return view('users.show', compact('user'));
    }

    // Hace el llamado a la función store para añadir un nuevo usuario
    public function create()
    {
        return view('users.create');
    }

    // Añade un nuevo usuarios a la DB
    public function store(){

        // recibe los campos del formulario y crea un usuario nuevo

        // $data = request()->all();   // recoge todos los campos son validar nada

        /*
         * Con los datos validados, el método validate() recibe un array asociativo con todas las validaciones,
         * se puede mandar otro array con los mensajes que se quieran mostar, donde las llaves son los nombres de los
         * campos.nombreValidación (name.required) y los valores son los mensajes
         */
        $data = request()->validate([
            'name' => 'required',
            'email' => 'required|email|unique:users,email', // Con | puedes tener más de una regla, o pasando un array ['required', 'email', '...']
            'password' => ['required','min:6'],
        ], [
            'name.required' => 'El campo "Nombre del usuario" es requerido',
            'email.required' => 'El campo "Email" es requerido',
            'email.unique' => 'Otro usuario stá usando el email, este debe ser único',
            'password.required' => 'El campo "Tu password" es requerido',
            'password.min' => 'El campo "Tu password" debe tener al menos 6 caracteres',
        ]);

        User::create([
            'name' => $data['name'],
            'email' => $data['email'],
            'password' => bcrypt($data['password']),
        ]);

        return redirect()->route('users.index');
    }

    // Hace llamado a la vista con el formulario para actualizar el usuario
    public function edit(User $user)
    {
        return view('users.edit', ['user' => $user]);
    }

    // Actualiza los usuarios
    public function update(User $user){
        $data = request()->validate([
            'name' => 'required',
            // Al actualizar se deja conservar el email del usuario
            // 'email' => 'required|email|unique:users,email,'.$user->id,
            // Con la nueva sintaxis de laravel Rule
            'email' => ['required', 'email', Rule::unique('users')->ignore($user->id)],
            'password' => '',   //  El password no tiene que ser requerido
        ], [
            'name.required' => 'El campo "Nombre del usuario" es requerido',
            'email.required' => 'El campo "Email" es requerido',
            'email.unique' => 'Otro usuario stá usando el email, este debe ser único',
            // 'password.required' => 'El campo "Tu password" es requerido',
            // 'password.min' => 'El campo "Tu password" debe tener al menos 6 caracteres',
        ]);

        /*
            Condicional que se encarga de mira el password, para que el usuario que esté actualizando datos no este obligado a cambiar su password
        */
        if ($data['password'] != null) {
            $data['password'] = bcrypt($data['password']);
        } else {
            unset($data['password']);   // Quita el indice password del array asociativo
        }

        $user->update($data);

        return redirect()->route('users.show', ['user' => $user]);
    }

    // Método para la eliminación de un usuario
    public function destroy(User $user){

        $user->delete();

        return redirect()->route('users.index');
    }

}
